<?php

/*
 * This is a PHP library that handles calling AWCAPTCHA.
 *    - Documentation and latest version
 *          http://publisher.advertway.pl/integrate-awcaptcha-with-your-website
 *    - Get a AWCAPTCHA API Key
 *          http://publisher.advertway.pl/registration
 *
 * Copyright (c) 2007 AdvertWayCAPTCHA -- http://advertway.pl
 *
 * version 1.0.1
 */

class captchaPlugIn
{
	/**
	 * Registry object
	 *
	 * @access	protected
	 * @var		object
	 */
	public $registry;
	
	/**
	 * Database object
	 *
	 * @access	protected
	 * @var		object
	 */
	public $DB;
	
	/**
	 * Settings object
	 *
	 * @access	protected
	 * @var		object
	 */
	public $settings;
	
	/**
	 * Request object
	 *
	 * @access	protected
	 * @var		object
	 */
	public $request;
	
	/**
	 * Language object
	 *
	 * @access	protected
	 * @var		object
	 */
	public $lang;
	
	/**
	 * Member object
	 *
	 * @access	protected
	 * @var		object
	 */
	public $member;
	
	/**
	 * Member data object
	 *
	 * @access	protected
	 * @var		object
	 */
	public $memberData;	
	
	/**
	 * Error code from reCAPTCHA
	 *
	 * @access	protected
	 * @var		string
	 */
	public $error;
	
	/**
	 * is captcha active
	 *
	 * @access	protected
	 * @var boolean
	 */
	public $active= 1;
	
	
	/**
	 * Constructor
	 *
	 * @access	public
	 * @param	object		Registry Object
	 * @return	void
	 */
	public function __construct( ipsRegistry $registry )
	{
		$this->registry = $registry;
		$this->DB       = $this->registry->DB();
		$this->settings =& $this->registry->fetchSettings();
		$this->request  =& $this->registry->fetchRequest();
		$this->lang     = $this->registry->getClass('class_localization');
		$this->member   = $this->registry->member();
		$this->memberData =& $this->registry->member()->fetchMemberData();
		
		/* Settings */
		$this->public_key	= trim( $this->settings['awcaptcha_publickey'] );
		$this->private_key	= trim( $this->settings['awcaptcha_privatekey'] );
		$this->awcaptcha = new AdvertWayApi($this->public_key, $this->private_key);
		
		$section= ipsRegistry::$current_section;
	    $this->active= 1;
	    
	    if ($section=='register' && $this->settings['awcaptcha_register']!=1) {
	        
	        $this->active= 0;
	    }
	    
	    if ($section=='lostpass' && $this->settings['awcaptcha_remind']!=1) {
	        
	        $this->active= 0;
	    }
	    
	    if ($section=='post' && $this->settings['awcaptcha_post']!=1) {
	        
	        $this->active= 0;
	    }
	    
	    if ($section=='login' && $this->settings['awcaptcha_login']!=1) {
	        
	        $this->active= 0;
	    }
	}
	
	/**
	 * Gets the challenge HTML (javascript and non-javascript version).
	 * This is called from the browser, and the resulting reCAPTCHA HTML widget
	 * is embedded within the HTML form it was called from.
	 *
	 * @access	public
	 * @return 	string		Form HTML to display
	 */
	public function getTemplate()
	{
	    if (!$this->active) {
	        
	        return false;
	    }
	    
        $html = $this->awcaptcha->getCaptchaWidget();
		return $html;
	}

	/**
	 * Validate the input code
	 *
	 * @access	public
	 * @return	boolean		Validation successful
	 * @since	1.0
	 */
	public function validate()
	{
	    if (!$this->active) {
	        
	        return true;
	    }
	
		$challenge	= $_REQUEST['captcha_challenge_code'];
		$solution	= $_REQUEST['captcha_solution'];
		
		if($challenge)
        {
            if(empty($solution))
            {
                $this->error = 'Kod jest niepoprawny';
                return FALSE;
            }
            else
            {
                if(!$this->awcaptcha->validate($challenge, $solution, $this->member->ip_address))
                {
                    $this->error = 'Kod jest niepoprawny';
                    return FALSE;
                    
                } else {
                    
                    return true;
                }
            }
        } else {
            
            return FALSE;
        }
	}
}



/**
 * @package   AdvertWay clientlib
 * @author    <wydawcy@advertway.pl> Advertway.pl
 * @license   http://www.gnu.org/copyleft/lesser.html  LGPL License 2.1
 * @link      http://publisher.advertway.pl/integrate-with-your-website
 * @version   1.0
 */

if ( !class_exists('AdvertWayApi') )
{

    /**
     * Klasa do obsługi systemu Advertway na stronach wydawców
     */
    class AdvertWayApi
    {
      protected $publicKey, $privateKey;
      protected $apiHost = 'api.advertway.pl';
      protected $apiPath = '/validate.php';
      protected $apiHostPort = 80;
      protected $errorMsg;
    
    
      public function __construct($publicKey, $privateKey, $apiHost = null)
      {
        $this->publicKey = $publicKey;
        $this->privateKey = $privateKey;
        if($apiHost)
          $this->apiHost = $apiHost;
      }
      
      /**
       * Wyświetla captchę na stronie
       */
      public function showCaptcha()
      {
        echo $this->getCaptchaWidget();
      }
      
      public function getCaptchaWidget()
      {
        return '<script type="text/javascript" src="http://' . $this->apiHost . '/get.php?public_key=' . $this->publicKey . '"></script>' . 
             '<noscript>
                <iframe src="http://' . $this->apiHost . '/getnoscript.php?public_key=' . $this->publicKey . '" width="330" height="310" frameborder="0" style="margin: 0"></iframe>
                <div><label>Skopiuj kod captchy:</label><input type="text" name="captcha_challenge_code" /></div>
                <div><label>Wpisz kod z obrazka:</label><input type="text" name="captcha_solution" /></div>
              </noscript>';
        
      }
      
      /**
       * Weryfikuje rozwiązanie captchy wyświetlonej na stronie
       * Zwraca true, gdy captcha została poprawnie rozwiązana. W innym przypadku zwraca false.
       * 
       * @param string $challengeCode identyfikator wyświetlonej captchy
       * @param string $captchaSolution rozwiązanie captchy wpisane przez użytkownika
       * @param string $userIp adres IP użytkownika, który rozwiązuje captche
       * @return boolean 
       */
      public function validate($challengeCode = null, $captchaSolution = null, $userIp = null)
      {
        //jeżeli nie został podany parametr $challengeCode, próbujemy go pobrać z pola o domyślnej nazwie 'captcha_challenge_code'
        if(!$challengeCode)
          $challengeCode = isset($_REQUEST['captcha_challenge_code']) ? $_REQUEST['captcha_challenge_code'] : '';
        
        //jeżeli nie został podany parametr $captchaSolution, próbujemy go pobrać z pola o domyślnej nazwie 'captcha_solution'
        if(!$captchaSolution)
          $captchaSolution = isset($_REQUEST['captcha_solution']) ? $_REQUEST['captcha_solution'] : '';
        
        //zmieniamy kodowanie znaków na utf-8
        $captchaSolution = $this->fixEncoding($captchaSolution);
        
        //jeżli nie został podany parametr $userIp, pobieramy adres korzystając ze zmiennej $_SERVER['REMOTE_ADDR']
        if(!$userIp)
          $userIp = $_SERVER['REMOTE_ADDR'];
    
        //usuwamy ewentualne slashe z przesłanego rozwiązania captchy
        $captchaSolution = get_magic_quotes_gpc() ? stripslashes($captchaSolution) : $captchaSolution;
        
        //tworzymy parametry do przesłania na serwer AdvertWay
        $data = 'public_key=' . $this->publicKey . '&challenge_code=' . $challengeCode . '&solution=' . $captchaSolution . '&user_ip=' . $userIp;
    
        //wysyłamy żądanie POST do serwera AdvertWay w celu weryfikacji poprawności rozwiązania
        $response = $this->sendHttpPost($data);
        
        //odpowiedź zawiera 3 pola: 
        // - rezultat weryfikacji
        // - komunikat błędu w przypadku negatywnej weryfikacji
        // - kod umożliwiający sprawdzenie autentyczności odpowiedzi serwera
        $r= new stdClass();
        list($r->result, $r->msg, $r->auth) = explode("\n", $response);
        
        //sprawdzamy autentyczność odpowiedzi serwera
        if($this->validateResponse($r->auth, $challengeCode, $captchaSolution))
        {
          //sprawdzamy jaki jest wynik weryfikacji captchy
          //w przypadku negatywnej weryfikacji zapisujemy komunikat błędu w polu $msg
          if(!$r->result)
            $this->errorMsg = $r->msg;
          
          return $r->result;
        }
        else
        {
          //gdy sprawdzanie autentyczności odpowiedzi serwera nie powiodło się zwracamy false
          //i zapisujemy komunikat błędu
          $this->errorMsg = 'Niepoprawna autoryzacja odpowiedzi';
          return false;
        }
      }
      
      /**
       * Funkcja zwraca komunikat błędu weryfikacji captchy
       * @return string
       */
      public function getErrorMsg()
      {
        return $this->errorMsg;
      }
    
      /**
       *
       * @param mixed $data parametry do wysłania do serwera weryfikującego
       * @return mixed 
       */
      protected function sendHttpPost($data)
      {
        $errno = $errstr = false;
        $response = '';    
        
        $httpRequest  = "POST $this->apiPath HTTP/1.0\r\n";
        $httpRequest .= "Host: $this->apiHost\r\n";
        $httpRequest .= "Content-Type: application/x-www-form-urlencoded\r\n";
        $httpRequest .= "Content-Length: " . strlen($data) . "\r\n";
        $httpRequest .= "\r\n";
        $httpRequest .= $data;
    
        $fs = @fsockopen($this->apiHost, $this->apiHostPort, $errno, $errstr, 10);
        if($fs == false)
          die('Could not open socket! ' . $errstr);
    
        fwrite($fs, $httpRequest);
    
        while(!feof($fs))
          $response .= fgets($fs);
        fclose($fs);
    
        $response = explode("\r\n\r\n", $response, 2);
    
        return $response[1];
      }
      
      /**
       * Metoda sprawdza autentyczność odpowiedzi serwera weryfikującego
       * 
       * @param string $authCode kod auth zwrócony przez serwer weryfikujący
       * @param string $challengeCode identyfikator rozwiązanej captchy
       * @param string $solution rozwiązanie captchy przesłane przez użytkownika
       * @return boolean 
       */
      protected function validateResponse($authCode, $challengeCode, $solution)
      {
        $calculatedAuth = sha1($challengeCode . $solution . $this->privateKey);
        return $authCode == $calculatedAuth ? true : false;
      }
      
      /**
       *
       * @param string $txt tekst, dla którego chcemy zmienić kodowanie
       * @return string 
       */
      protected function fixEncoding($txt)
      {
        $encoding = mb_detect_encoding($txt, "UTF-8, ISO-8859-2, ISO-8859-1, ISO-8859-15", true);
        
        if($encoding)
          $txt = mb_convert_encoding($txt, "UTF-8", $encoding);
        
        return $txt;
      }
    }
}